require_relative '../awslive-common/action_type'
require_relative '../awslive-common/start_type'
require_relative 'input_switch_schedule_action'
require_relative 'pause_schedule_action'
require_relative 'ad_marker_scheduled_action'
require_relative 'input_prepare_schedule_action'
require 'uri'
require 'cgi'

require 'aws-sdk-medialive'

module Awslive
  class ScheduleAction
    include ActionType
    include StartType

    def initialize(mediaclient = nil)
      if !mediaclient.nil?
        @mediaclient = mediaclient
      else
        credentials = Aws::SharedCredentials.new
        @mediaclient = Aws::MediaLive::Client.new(:credentials => credentials)
      end
      @input_switch_schedule_action = Awslive::InputSwitchScheduleAction.new
      @pause_schedule_action = Awslive::PauseScheduleAction.new
      @ad_marker_schedule_action = Awslive::AdMarkerScheduledAction.new
      @input_prepare_schedule_action = Awslive::InputPrepareScheduleAction.new
    end

    def create_schedule(channel_id, scheduled_actions )
      payload = {
          channel_id: "#{channel_id}",
          creates: {
              schedule_actions: scheduled_actions
          }
      }
      @mediaclient.batch_update_schedule( payload )
    end

    def delete(channel_id, action_names)
      @mediaclient.batch_update_schedule( {
                                              channel_id: "#{channel_id}",
                                              deletes: {
                                                  action_names: action_names
                                              }
                                          } )

    end

    def get_follow_schedule(channel_id, action_name)
      puts "Follow for action Name : #{action_name}"
      follow_schedule = nil
      response = @mediaclient.describe_schedule({channel_id: "#{channel_id}", max_results: 100 } )
      loop do
        response[:schedule_actions].each do | schedule_action |
          unless schedule_action[:schedule_action_start_settings][:follow_mode_schedule_action_start_settings].nil?
            # This is follow action
            if schedule_action[:schedule_action_start_settings][:follow_mode_schedule_action_start_settings][:reference_action_name] == action_name
              # Identifying only the input switch follow action for right follow up! as it provides SCTE35 follow up as well.
              puts "Scheduled Action #{schedule_action}"
              if schedule_action[:schedule_action_settings][:input_switch_settings].nil?
                next
              end
              follow_schedule = schedule_action
              break
            end
          end
        end
        break if !follow_schedule.nil? || response[:next_token].nil?
        response = @mediaclient.describe_schedule({channel_id: "#{channel_id}", max_results: 100, next_token: response[:next_token] } )
      end
      follow_schedule
    end

    def get_type_duration(url)
      type = nil
      duration = nil
      if !url.nil?
        uri = URI.parse(url) rescue nil
        params = CGI.parse(uri.query) unless uri.nil?
        if !params.nil?
          type = params["type"].first rescue nil
          duration = params["duration"].first rescue nil
        end
      end
      return type, duration
    end

    def create(channel_id, start_type, action_type, data)
      scheduled_action = []
      ad_scheduled_action = []
      create_response = []

      if action_type == INPUT_SWITCH
        if start_type == IMMEDIATE
          scheduled_action << @input_switch_schedule_action.get_immediate_schedule_action(data[:input_ref_name], data[:url_path])
          type, duration = get_type_duration(data[:url_path])
          if !type.nil? && !duration.nil? && type == "ad_marker"
            ad_scheduled_action << @ad_marker_schedule_action.get_immediate_schedule_action(duration)
          end
        elsif start_type == FOLLOW
          scheduled_action << @input_switch_schedule_action.get_follow_schedule_action(data[:follow_input], data[:trigger_input], data[:url_path])
          type, duration = get_type_duration(data[:url_path])
          if !type.nil? && !duration.nil? && type == "ad_marker"
            puts "ad marker provisioned for duration => #{duration}"
            ad_scheduled_action << @ad_marker_schedule_action.get_follow_schedule_action(data[:follow_input], duration.to_i)
          end
        end
      elsif action_type == PAUSE
        if start_type == FIXED
          scheduled_action << @pause_schedule_action.get_pause_schedule_action(data[:channel_class])
        end
      elsif action_type == UNPAUSE
        if start_type == FIXED
          scheduled_action << @pause_schedule_action.get_unpause_schedule_action
        end
      elsif action_type == INPUT_PREPARE
        if start_type == IMMEDIATE
          scheduled_action << @input_prepare_schedule_action.get_immediate_schedule_action(data[:input_ref_name])
        end
      end



      unless scheduled_action.empty?
        create_response << create_schedule(channel_id, scheduled_action)
        unless ad_scheduled_action.empty?
          puts "Ad Schedule Action #{ad_scheduled_action}"
          create_response << create_schedule(channel_id, ad_scheduled_action) rescue nil
        end
      end

      create_response
    end

    def get_schedule_action(channel_id, action_name)
      sc_action = nil
      response = @mediaclient.describe_schedule({
                                                    channel_id: "#{channel_id}", max_results: 100
                                                })
      loop do
        response[:schedule_actions].each do | schedule_action |
          if schedule_action[:action_name] == action_name
            sc_action = schedule_action
            break
          end
        end
        break if !sc_action.nil? || response[:next_token].nil?
        response = @mediaclient.describe_schedule({channel_id: "#{channel_id}", max_results: 100, next_token: response[:next_token] } )
      end
      sc_action
    end

    def get_schedule_action_type(schedule_action)
      action_type = nil
      schedule_action[:schedule_action_settings].each_pair do | key, value |
        if !value.nil?
          action_type = key.to_s.sub!("_settings","")
          break
        end
      end
      action_type
    end

    def get_schedule_start_type(schedule_action)
      start_type = nil
      schedule_action[:schedule_action_start_settings].each_pair do | key, value |
        if !value.nil?
          start_type = key.to_s.sub!("_mode_schedule_action_start_settings","")
          break
        end
      end
      start_type
    end

    def get_schedule_actions(channel_id, action_type = nil, start_type = nil)
      sc_actions = []
      response = @mediaclient.describe_schedule({
                                                    channel_id: "#{channel_id}", max_results: 100
                                                })
      loop do
        response[:schedule_actions].each do | schedule_action |
          if !action_type.nil?
            current_action_type = get_schedule_action_type(schedule_action)
            #puts "Action Type #{current_action_type}  : #{action_type}"
            next if current_action_type != action_type.to_s
          end
          if !start_type.nil?
            current_start_type = get_schedule_start_type(schedule_action)
            #puts "start Type #{current_start_type}  :  #{start_type}"
            next if current_start_type.to_s != start_type.to_s
          end
          sc_actions << schedule_action
        end
        break if response[:next_token].nil?
        response = @mediaclient.describe_schedule({channel_id: "#{channel_id}", max_results: 100, next_token: response[:next_token] } )
      end
      sc_actions
    end

    def get_follow_action_url(channel_id, action_name)
      url = nil
      follow_schedule = get_follow_schedule(channel_id, action_name)
      url = follow_schedule[:schedule_action_settings][:input_switch_settings][:url_path] unless follow_schedule.nil?
      url
    end

    def get_action_url(channel_id, action_name)
      url = nil
      follow_schedule = get_schedule_action(channel_id, action_name)
      url = follow_schedule[:schedule_action_settings][:input_switch_settings][:url_path] unless follow_schedule.nil?
      url
    end

    def get_action_id
      Time.now.to_i
    end

    def delete_all(channel_id)
      @mediaclient.delete_schedule({ channel_id: channel_id })
    end
  end
end